package DTDDoc;

import java.io.*;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.Iterator;

import org.apache.tools.ant.taskdefs.MatchingTask;
import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.DirectoryScanner;
import org.apache.tools.ant.types.FileSet;

/** This is the DTDDoc ant task. */

public class DTDDocTask extends MatchingTask {

    private boolean showHiddenTags = false;
    public void setShowHiddenTags( boolean f) {
	    showHiddenTags = f;
	}

	private boolean showFixmeTags = true;
    public void setShowFixmeTags( boolean f) {
	    showFixmeTags = f;
	}

    private boolean getAroundNetBeansComments = true;
    public void setGetAroundNetBeansComments( boolean f) {
	    getAroundNetBeansComments = f;
	}


    /** Track if the "sourceDir" task parameter was set or not in
     *  the Ant script. */
    private boolean sourceDirSet = false;

	private File sourceDir = new File("."); // FIXME This is not portable
    public void setSourceDir( File f) {
	    sourceDir = f;
        sourceDirSet = true;
    }

    /**
     * Pseudo-parameter addition to mimic FileSet definition with the real
     * task parameter : sourceDir.
     * @param dir
     */
    public void setDir( File dir) {
    	setSourceDir( dir);
    }

    public void setFile( File file) {
    	setSourceDir( file.getParentFile());
    	setIncludes( file.getName());
    }

	private File destDir = new File("."); // FIXME This is not portable
    public void setDestDir( File f) {
	    destDir = f;
	}

	private String docTitle = "";
    public void setDocTitle( String s) {
	    docTitle = s;
	}

	private File styleSheet = null;
	public void setStyleSheet( File f) {
		if( f.canRead()) {
			styleSheet = f;
		} else {
			log.warn("Sorry, the style sheet '"+f+"' was not" +
				" found. I'll use the default one instead.");
		}
	}


    /** Filesets as explained in Ant's documentation. */
    private List filesets = new ArrayList();

    /** Filesets as explained in Ant's documentation. */
    public void addFileset( FileSet fileset) {
        filesets.add( fileset);
    }

	private Logger log;

	public DTDDocTask() {
		log = new AntLogger( this);
	}

    /** Scan a directory with given scanner and inject the list of included
     * Files (complete paths) in a given set.
     *
     * @param scanner DirectoryScanner to use.
     * @param sDir Base directory where the scanner will go.
     * @param hfiles Set where to put the path of the included files. */

    private void scanDirectory( DirectoryScanner scanner, File sDir, Set hfiles) {
        String[] files = scanner.getIncludedFiles();

        for (int i = 0; i < files.length; i++) {
		    hfiles.add( new File( sDir, files[i]));
	    }
    }

    public void execute() throws BuildException {

	    log.info("Welcome to DTDDoc " + DTDCommenter.VERSION + ".");
	    log.info("Parts of this program are from DTDParser" +
	    	" and RegExp of the Apache Project.");
	    log.info("The settings are :");

		log.info( "   " + (showHiddenTags ? "Show" : "Don't show") + " @hidden tags.");
	    log.info( "   " + (showFixmeTags ? "Show" : "Don't show") + " @fixme tags.");


		if( styleSheet != null) {
			log.info("   Style sheet source is '"+styleSheet.getPath()+"'.");
		} else {
			log.info("   No style sheet was specified.");
		}

		try {
			log.info("   Source directory is '" +
				sourceDir.getCanonicalPath() + "'.");
			log.info("   Destination directory is '" +
				destDir.getCanonicalPath() + "'.");
		} catch ( IOException ex) {
			log.error("Something went wrong while figuring out" +
				" the source/dest directories paths.", ex);
			return;
		}

        // Now we locate the files according to the scanner...

		DirectoryScanner scanner = getDirectoryScanner( sourceDir );
		String[] files = scanner.getIncludedFiles();

		if( files.length == 0)
			log.error(" No files to process ! Please note that you" +
				" have to specify which files to include with the <include> tags. There's no" +
				" automatic selection of all DTD's anymore (as of 0.0.7).");

		Set scan = new HashSet();

		if( filesets.size() >  0) {
            // Here we use a set of <filesets>

            if( sourceDirSet)
                log.warn("When using filesets, the \"dir\" attribute of " +
                    "the file set has precedence over the \"sourceDir\" " +
                    "of the task." );
            for(Iterator itFSets = filesets.iterator(); itFSets.hasNext(); ) {
                FileSet fs = (FileSet)itFSets.next();
                scanDirectory( fs.getDirectoryScanner( getProject() ),
                    fs.getDir( getProject()), scan);
            }
        } else {
            // Here we use a combination of <includes> and <excludes>
            scanDirectory( getDirectoryScanner( sourceDir ), sourceDir, scan);
        }

		try {
            DTDCommenter commenter = new DTDCommenter( log);
		    commenter.commentDTDs( scan,
			                       sourceDir,
								   destDir,
								   showHiddenTags, showFixmeTags,
                                   getAroundNetBeansComments, docTitle, styleSheet);
		} catch ( Exception ex) {
		    log.error("A problem occured while instanciating or " +
                "executing the DTDDoc task !", ex);
		    return;
		}
	}
}
