/*
 * Created on 1-jul.-2006 */
package DTDDoc;


/** A logger class that should be compatible with Ant and Maven's log.
 *
 * This class was introduced to abstract the details of Ant and Maven's
 * logging. This is needed because when DTDDoc works as an Ant task
 * we use Ant for logs but when we're under Maven we use Maven's log.
 *
 * @author Stefan Champailler schampailler_at_skynet_dot_be */


public abstract class Logger {

    /** Logs an informational message. Such a message never indicates
	 *  a problem to the user.
	 *
	 * @param msg Message to log. */
	 
    public abstract void info( String msg);

    /** Logs a warning message. Such a message indicates a problem
	 *  to the user. The problem doesn't affect the course of 
	 *  DTDDoc but is worth noting because it's the result of a
	 *  an action or the data of the user.
	 *
	 * @param msg Message to log. */
	 
    public abstract void warn( String msg);
	
    /** Logs an error message. Such a message indicates something
	 *  wrong happened inside DTDDoc. This thing may affect the
	 *  course of execution of DTDDoc. That is, if it happens, the
	 *  result of the execution is not guaranteed.
	 *
	 * @param msg Message to log. */
	
	public abstract void error( String msg);
	
	/** Logs an error message and an associated exception (stack trace).
	 *
	 *  @see #error( String msg)
	 *  @param msg Message to log.
	 *  @param ex Exception to log. */
	 
    public void error( String msg, Exception ex) {
        error( msg);
		error( ex.toString());
    }

}

